/*++ 
 * linux/drivers/video/wmt/hw/wmt-govrh-reg.h
 * WonderMedia video post processor (VPP) driver
 *
 * Copyright c 2010  WonderMedia  Technologies, Inc.
 *
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * WonderMedia Technologies, Inc.
 * 4F, 533, Chung-Cheng Road, Hsin-Tien, Taipei 231, R.O.C
--*/

/*
 * ChangeLog
 *
 * 2010-10-06  Sam Shen
 *     * create for wm8710 project
 */

/* Product ID: 8650 / Project ID: 3465 */

#ifndef WMT_GOVRH_REG_H
#define WMT_GOVRH_REG_H

// feature
#define WMT_FTBLK_GOVRH
#define WMT_FTBLK_GOVRH_IGS
#define WMT_FTBLK_GOVRH_CURSOR
// #define WMT_FTBLK_GOVRH_VGA
// #define WMT_FTBLK_GOVRH_DAC
// #define WMT_FTBLK_GOVRH_DAC_THREE

//Base1
#define REG_GOVRH_BASE1_BEGIN		(GOVRH_BASE1_ADDR+0x00)
#define REG_GOVRH_CUR_ADDR			(GOVRH_BASE1_ADDR+0x00)
#define REG_GOVRH_CUR_WIDTH			(GOVRH_BASE1_ADDR+0x04)
#define REG_GOVRH_CUR_FB_WIDTH		(GOVRH_BASE1_ADDR+0x08)
#define REG_GOVRH_CUR_VCROP			(GOVRH_BASE1_ADDR+0x0C)
#define REG_GOVRH_CUR_HCROP			(GOVRH_BASE1_ADDR+0x10)
#define REG_GOVRH_CUR_HCOORD		(GOVRH_BASE1_ADDR+0x14)
#define REG_GOVRH_CUR_VCOORD		(GOVRH_BASE1_ADDR+0x18)
#define REG_GOVRH_CUR_STATUS		(GOVRH_BASE1_ADDR+0x1C)
#define REG_GOVRH_CUR_COLOR_KEY		(GOVRH_BASE1_ADDR+0x20)
#define REG_GOVRH_DVO_PIX			(GOVRH_BASE1_ADDR+0x30)
#define REG_GOVRH_DVO_DLY_SEL		(GOVRH_BASE1_ADDR+0x34)
#define REG_GOVRH_INT				(GOVRH_BASE1_ADDR+0x38)
#define REG_GOVRH_DVO_BLANK_DATA	(GOVRH_BASE1_ADDR+0x3C)
#define REG_GOVRH_MIF				(GOVRH_BASE1_ADDR+0x80)
#define REG_GOVRH_COLFMT			(GOVRH_BASE1_ADDR+0x84)	//422,420
#define REG_GOVRH_SRCFMT			(GOVRH_BASE1_ADDR+0x88)
#define REG_GOVRH_DSTFMT			(GOVRH_BASE1_ADDR+0x8C)
#define REG_GOVRH_YSA				(GOVRH_BASE1_ADDR+0x90)
#define REG_GOVRH_CSA				(GOVRH_BASE1_ADDR+0x94)
#define REG_GOVRH_PIXWID			(GOVRH_BASE1_ADDR+0x98)
#define REG_GOVRH_BUFWID			(GOVRH_BASE1_ADDR+0x9c)
#define REG_GOVRH_VCROP				(GOVRH_BASE1_ADDR+0xa0)
#define REG_GOVRH_HCROP				(GOVRH_BASE1_ADDR+0xa4)
#define REG_GOVRH_FHI				(GOVRH_BASE1_ADDR+0xa8)
#define REG_GOVRH_COLFMT2			(GOVRH_BASE1_ADDR+0xac)	//444
#define REG_GOVRH_REG_STS			(GOVRH_BASE1_ADDR+0xe4)
#define REG_GOVRH_SWFLD				(GOVRH_BASE1_ADDR+0xe8)
#define REG_GOVRH_BASE1_END			(GOVRH_BASE1_ADDR+0xe8)

//Base2
#define REG_GOVRH_BASE2_BEGIN		(GOVRH_BASE2_ADDR+0x00)
#define REG_GOVRH_TG_ENABLE			(GOVRH_BASE2_ADDR+0x00)	//TG
#define REG_GOVRH_READ_CYC			(GOVRH_BASE2_ADDR+0x04)
#define REG_GOVRH_H_ALLPXL			(GOVRH_BASE2_ADDR+0x08)	//TG_1
#define REG_GOVRH_V_ALLLN			(GOVRH_BASE2_ADDR+0x0c)
#define REG_GOVRH_ACTLN_BG			(GOVRH_BASE2_ADDR+0x10)
#define REG_GOVRH_ACTLN_END			(GOVRH_BASE2_ADDR+0x14)
#define REG_GOVRH_ACTPX_BG			(GOVRH_BASE2_ADDR+0x18)
#define REG_GOVRH_ACTPX_END			(GOVRH_BASE2_ADDR+0x1c)
#define REG_GOVRH_VBIE_LINE			(GOVRH_BASE2_ADDR+0x20)
#define REG_GOVRH_PVBI_LINE			(GOVRH_BASE2_ADDR+0x24)
#define REG_GOVRH_HDMI_VBISW		(GOVRH_BASE2_ADDR+0x28)
#define REG_GOVRH_HDMI_HSYNW		(GOVRH_BASE2_ADDR+0x2c)
#define REG_GOVRH_VSYNC_OFFSET		(GOVRH_BASE2_ADDR+0x30)
#define REG_GOVRH_FIELD_STATUS		(GOVRH_BASE2_ADDR+0x34)
#define REG_GOVRH_DVO_SET			(GOVRH_BASE2_ADDR+0x48)
#define REG_GOVRH_CB_ENABLE			(GOVRH_BASE2_ADDR+0x50)
#define REG_GOVRH_H_ALLPXL2			(GOVRH_BASE2_ADDR+0x58)
#define REG_GOVRH_V_ALLLN2			(GOVRH_BASE2_ADDR+0x5c)
#define REG_GOVRH_ACTLN_BG2			(GOVRH_BASE2_ADDR+0x60)
#define REG_GOVRH_ACTLN_END2		(GOVRH_BASE2_ADDR+0x64)
#define REG_GOVRH_ACTPX_BG2			(GOVRH_BASE2_ADDR+0x68)
#define REG_GOVRH_ACTPX_END2		(GOVRH_BASE2_ADDR+0x6c)
#define REG_GOVRH_VBIE_LINE2		(GOVRH_BASE2_ADDR+0x70)
#define REG_GOVRH_PVBI_LINE2		(GOVRH_BASE2_ADDR+0x74)
#define REG_GOVRH_HDMI_VBISW2		(GOVRH_BASE2_ADDR+0x78)
#define REG_GOVRH_HDMI_HSYNW2		(GOVRH_BASE2_ADDR+0x7c)
#define REG_GOVRH_CONTRAST			(GOVRH_BASE2_ADDR+0xb8)
#define REG_GOVRH_BRIGHTNESS		(GOVRH_BASE2_ADDR+0xbc)
#define REG_GOVRH_DMACSC_COEF0		(GOVRH_BASE2_ADDR+0xc0)
#define REG_GOVRH_DMACSC_COEF1		(GOVRH_BASE2_ADDR+0xc4)
#define REG_GOVRH_DMACSC_COEF2		(GOVRH_BASE2_ADDR+0xc8)
#define REG_GOVRH_DMACSC_COEF3		(GOVRH_BASE2_ADDR+0xcc)
#define REG_GOVRH_DMACSC_COEF4		(GOVRH_BASE2_ADDR+0xd0)
#define REG_GOVRH_DMACSC_COEF5		(GOVRH_BASE2_ADDR+0xd8)
#define REG_GOVRH_DMACSC_COEF6		(GOVRH_BASE2_ADDR+0xdc)
#define REG_GOVRH_CSC_MODE			(GOVRH_BASE2_ADDR+0xe0)
#define REG_GOVRH_YUV2RGB			(GOVRH_BASE2_ADDR+0xe4)
#define REG_GOVRH_H264_INPUT_EN		(GOVRH_BASE2_ADDR+0xe8)
#define REG_GOVRH_HSCALE_UP			(GOVRH_BASE2_ADDR+0xF4)
#define REG_GOVRH_IGS_MODE			(GOVRH_BASE2_ADDR+0xF8)
#define REG_GOVRH_IGS_MODE2			(GOVRH_BASE2_ADDR+0xFC)
#define REG_GOVRH_BASE2_END			(GOVRH_BASE2_ADDR+0xFC)

//Base1
// REG_GOVRH_CUR_ADDR,0x00
// REG_GOVRH_CUR_WIDTH,0x04
#define GOVRH_CUR_WIDTH				REG_GOVRH_CUR_WIDTH,0x7F,0

// REG_GOVRH_CUR_FB_WIDTH,0x08
#define GOVRH_CUR_FB_WIDTH			REG_GOVRH_CUR_FB_WIDTH,0x7FF,0

// REG_GOVRH_CUR_VCROP,0x0C
#define GOVRH_CUR_VCROP				REG_GOVRH_CUR_VCROP,0x7F,0

// REG_GOVRH_CUR_HCROP,0x10
#define GOVRH_CUR_HCROP				REG_GOVRH_CUR_HCROP,0x7F,0

// REG_GOVRH_CUR_HCOORD,0x14
#define GOVRH_CUR_H_END				REG_GOVRH_CUR_HCOORD,0x7FF0000,16
#define GOVRH_CUR_H_START			REG_GOVRH_CUR_HCOORD,0x7FF,0

// REG_GOVRH_CUR_VCOORD,0x18
#define GOVRH_CUR_V_END				REG_GOVRH_CUR_VCOORD,0x7FF0000,16
#define GOVRH_CUR_V_START			REG_GOVRH_CUR_VCOORD,0x7FF,0

// REG_GOVRH_CUR_STATUS,0x1C
#define GOVRH_CUR_OUT_FIELD			REG_GOVRH_CUR_STATUS,BIT8,8
#define GOVRH_CUR_ENABLE			REG_GOVRH_CUR_STATUS,BIT0,0

// REG_GOVRH_CUR_COLOR_KEY,x20
#define GOVRH_CUR_ALPHA_ENABLE		REG_GOVRH_CUR_COLOR_KEY,BIT28,28
#define GOVRH_CUR_COLKEY_INVERT		REG_GOVRH_CUR_COLOR_KEY,BIT25,25
#define GOVRH_CUR_COLKEY_ENABLE		REG_GOVRH_CUR_COLOR_KEY,BIT24,24
#define GOVRH_CUR_COLKEY			REG_GOVRH_CUR_COLOR_KEY,0xFFFFFF,0

//REG_GOVRH_DVO_PIX,0x30
#define GOVRH_DVO_YUV422			REG_GOVRH_DVO_PIX,BIT1,1
#define GOVRH_DVO_RGB				REG_GOVRH_DVO_PIX,BIT0,0

//REG_GOVRH_DVO_DLY_SEL,0x34
#define GOVRH_DVO_CLK_INV			REG_GOVRH_DVO_DLY_SEL,BIT14,14
#define GOVRH_DVO_CLK_DLY			REG_GOVRH_DVO_DLY_SEL,0x3FFF,0
	
//REG_GOVRH_INT,0x38
#define GOVRH_INT_MEM				REG_GOVRH_INT,BIT17,17
#define GOVRH_INT_CUR				REG_GOVRH_INT,BIT16,16
#define GOVRH_INT_MEM_ENABLE		REG_GOVRH_INT,BIT1,1
#define GOVRH_INT_CUR_ENABLE		REG_GOVRH_INT,BIT0,0

//REG_GOVRH_DVO_BLANK_DATA,0x3C
#define GOVRH_DVO_BLANK_DATA		REG_GOVRH_DVO_BLANK_DATA,0x3FFFFFFF

//REG_GOVRH_MIF,0x80
#define GOVRH_H264_FMT				REG_GOVRH_MIF,BIT8,8
#define GOVRH_MIF_ENABLE			REG_GOVRH_MIF,BIT0,0

//REG_GOVRH_COLFMT,0x84
#define GOVRH_COLFMT				REG_GOVRH_COLFMT,BIT0,0

//REG_GOVRH_INFMT,0x88
#define GOVRH_INFMT					REG_GOVRH_SRCFMT,BIT0,0	// 0-frame, 1-field

//REG_GOVRH_DSTFMT,0x8C
#define GOVRH_OUTFMT				REG_GOVRH_DSTFMT,BIT0,0	// 0-frame, 1-field

//REG_GOVRH_YSA,0x90

//REG_GOVRH_CSA,0x94

//REG_GOVRH_AWIDTH,0x98
#define GOVRH_AWIDTH				REG_GOVRH_PIXWID, 0x7FF, 0

//REG_GOVRH_FWIDTH,0x9c
#define GOVRH_FWIDTH				REG_GOVRH_BUFWID, 0x7FF, 0

//REG_GOVRH_VCROP,0xa0
#define GOVRH_VCROP					REG_GOVRH_VCROP, 0x7FF, 0

//REG_GOVRH_HCROP,0xa4
#define GOVRH_HCROP					REG_GOVRH_HCROP, 0x7FF, 0

//REG_GOVRH_FHI,0xa8
#define GOVRH_FIFO_IND				REG_GOVRH_FHI, 0xF, 0

//REG_GOVRH_COLFMT2,0xac
#define GOVRH_COLFMT2				REG_GOVRH_COLFMT2,BIT0,0

//REG_GOVRH_REG_STS,0xe4
#define GOVRH_REG_LEVEL				REG_GOVRH_REG_STS,BIT8,8
#define GOVRH_REG_UPDATE			REG_GOVRH_REG_STS,BIT0,0

//REG_GOVRH_SWFLD,0xe8
#define GOVRH_SWFLD_ENABLE			REG_GOVRH_SWFLD,BIT1,1
#define GOVRH_SWFLD_FIXED			REG_GOVRH_SWFLD,BIT0,0

//Base2
//REG_GOVRH_TG_ENABLE,0x00
#define GOVRH_TG_MODE				REG_GOVRH_TG_ENABLE,BIT8,8
#define GOVRH_TG_ENABLE				REG_GOVRH_TG_ENABLE,BIT0,0

//REG_GOVRH_READ_CYC,0x04
#define GOVRH_READ_CYC				REG_GOVRH_READ_CYC, 0x7F, 0

//REG_GOVRH_H_ALLPXL,0x08
#define GOVRH_H_ALLPXL				REG_GOVRH_H_ALLPXL, 0xFFF, 0

//REG_GOVRH_V_ALLLN,0x0c
#define GOVRH_V_ALLLN				REG_GOVRH_V_ALLLN, 0xFFF, 0

//REG_GOVRH_ACTLN_BG,0x10
#define GOVRH_ACTLN_BG				REG_GOVRH_ACTLN_BG, 0xFFF, 0

//REG_GOVRH_ACTLN_END,0x14
#define GOVRH_ACTLN_END				REG_GOVRH_ACTLN_END, 0xFFF, 0

//REG_GOVRH_ACTPX_BG,0x18
#define GOVRH_ACTPX_BG				REG_GOVRH_ACTPX_BG, 0xFFF, 0

//REG_GOVRH_ACTPX_END,0x1c
#define GOVRH_ACTPX_END				REG_GOVRH_ACTPX_END, 0xFFF, 0

//REG_GOVRH_VBIE_LINE,0x20
#define GOVRH_VBIE_LINE				REG_GOVRH_VBIE_LINE, 0x7F, 0

//REG_GOVRH_PVBI_LINE,0x24
#define GOVRH_PVBI_LINE				REG_GOVRH_PVBI_LINE, 0x1F, 0

//REG_GOVRH_HDMI_VBISW,0x28
#define GOVRH_HDMI_VBISW			REG_GOVRH_HDMI_VBISW, 0xFF, 0

//REG_GOVRH_HDMI_HSYNW,0x2c
#define GOVRH_HDMI_HSYNW			REG_GOVRH_HDMI_HSYNW, 0xFF, 0

// REG_GOVRH_VSYNC_OFFSET,0x30
#define GOVRH_FIELD_INVERT			REG_GOVRH_VSYNC_OFFSET,BIT16,16
#define GOVRH_VSYNC_OFFSET			REG_GOVRH_VSYNC_OFFSET,0xFFF,0

// REG_GOVRH_FIELD_STATUS,0x34
#define GOVRH_FIELD_STATUS			REG_GOVRH_FIELD_STATUS,BIT0,0	// 1-BOTTOM,0-TOP

//REG_GOVRH_DVO_SET,0x48
#define GOVRH_DVO_BLK_DIS			REG_GOVRH_DVO_SET,BIT16,16
#define GOVRH_RGB_SWAP				REG_GOVRH_DVO_SET,0x300,8
#define GOVRH_DVO_VSYNC_POLAR		REG_GOVRH_DVO_SET,BIT3,3
#define GOVRH_DVO_ENABLE			REG_GOVRH_DVO_SET,BIT2,2
#define GOVRH_DVO_SYNC_POLAR		REG_GOVRH_DVO_SET,BIT1,1
#define GOVRH_DVO_OUTWIDTH			REG_GOVRH_DVO_SET,BIT0,0

//REG_GOVRH_CB_ENABLE,0x50
#define GOVRH_CB_INVERSION			REG_GOVRH_CB_ENABLE,BIT16,16
#define GOVRH_CB_MODE				REG_GOVRH_CB_ENABLE,BIT8,8
#define GOVRH_CB_ENABLE				REG_GOVRH_CB_ENABLE,BIT0,0

//REG_GOVRH_H_ALLPXL2,0x58
#define GOVRH_H_ALLPXL2				REG_GOVRH_H_ALLPXL2, 0xFFF, 0

//REG_GOVRH_V_ALLLN2,0x5c
#define GOVRH_V_ALLLN2				REG_GOVRH_V_ALLLN2,0xFFF,0

//REG_GOVRH_ACTLN_BG2,0x60
#define GOVRH_ACTLN_BG2				REG_GOVRH_ACTLN_BG2, 0xFF, 0

//REG_GOVRH_ACTLN_END2,0x64
#define GOVRH_ACTLN_END2			REG_GOVRH_ACTLN_END2, 0x7FF, 0

//REG_GOVRH_ACTPX_BG2,0x68
#define GOVRH_ACTPX_BG2				REG_GOVRH_ACTPX_BG2, 0x1FF, 0

//REG_GOVRH_ACTPX_END2,0x6c
#define GOVRH_ACTPX_END2			REG_GOVRH_ACTPX_END2, 0x1FFF, 0

//REG_GOVRH_VBIE_LINE2,0x70
#define GOVRH_VBIE_LINE2			REG_GOVRH_VBIE_LINE2, 0x7F, 0

//REG_GOVRH_PVBI_LINE2,0x74
#define GOVRH_PVBI_LINE2			REG_GOVRH_PVBI_LINE2, 0x1F, 0

//REG_GOVRH_HDMI_VBISW2,0x78
#define GOVRH_HDMI_VBISW2			REG_GOVRH_HDMI_VBISW2, 0xFF, 0

//REG_GOVRH_HDMI_HSYNW2,0x7c
#define GOVRH_HDMI_HSYNW2			REG_GOVRH_HDMI_HSYNW2, 0xFF, 0

//REG_GOVRH_CONTRAST,0xb8
#define GOVRH_CONTRAST_YAF			REG_GOVRH_CONTRAST, 0xFF0000, 16
#define GOVRH_CONTRAST_PBAF			REG_GOVRH_CONTRAST, 0xFF00, 8
#define GOVRH_CONTRAST_PRAF			REG_GOVRH_CONTRAST, 0xFF, 0

//REG_GOVRH_BRIGHTNESS,0xbc
#define GOVRH_BRIGHTNESS_Y			REG_GOVRH_BRIGHTNESS, 0xFF, 0

// 0xC0 - 0xDC CSC

//REG_GOVRH_CSC_MODE,0xe0
#define GOVRH_CSC_MOD				REG_GOVRH_CSC_MODE,BIT0,0
#define GOVRH_CSC_CLAMP_ENABLE		REG_GOVRH_CSC_MODE,BIT1,1

//REG_GOVRH_DVO_YUV2RGB,0xe4
#define GOVRH_DVO_YUV2RGB_ENABLE	REG_GOVRH_YUV2RGB,BIT0,0
#define GOVRH_DAC_CLKINV			REG_GOVRH_YUV2RGB,BIT3,3
#define GOVRH_BLANK_ZERO			REG_GOVRH_YUV2RGB,BIT4,4
#define GOVRH_RGB_MODE				REG_GOVRH_YUV2RGB,0x300,8	// 0-YUV, 1-RGB24, 2-1555, 3-565

//REG_GOVRH_H264_INPUT_EN, 0xe8
#define GOVRH_H264_INPUT_ENABLE		REG_GOVRH_H264_INPUT_EN,BIT0,0

// REG_GOVRH_IGS_MODE,0xF8
#define GOVRH_LDI_MODE				REG_GOVRH_IGS_MODE,BIT8,8
#define GOVRH_IGS_MODE				REG_GOVRH_IGS_MODE,0x7,0

// REG_GOVRH_IGS_MODE2,0xFC
#define GOVRH_LDI_MODE2				REG_GOVRH_IGS_MODE2,BIT8,8
#define GOVRH_IGS_MODE2				REG_GOVRH_IGS_MODE2,0x7,0

#endif				/* WMT_GOVRH_REG_H */
